@@ -10,7 +10,7 @@ from devpi.use import Current
 import devpi.server
 import requests
 import json
-from devpi.server import ensure_autoserver
+from devpi.server import handle_autoserver
 std = py.std
 subcommand = lazydecorator()
 
@@ -56,6 +56,10 @@ class Hub:
         self._tw = py.io.TerminalWriter()
         self.args = args
         self.cwd = py.path.local()
+        self.quiet = False
+
+    def set_quiet(self):
+        self.quiet = True
 
     @property
     def clientdir(self):
@@ -63,7 +67,7 @@ class Hub:
 
     def require_valid_current_with_index(self):
         if not self.current.simpleindex:
-            self.fatal("need to be using a live index, see 'devpi use'")
+            handle_autoserver(self, self.current)
         return self.current
 
 
@@ -113,10 +117,13 @@ class Hub:
 
     def update_auth(self, user, password):
         self.http.auth = (user, password)
+        self.clientdir.ensure(dir=1)
         oldumask = os.umask(7*8+7)
-        self.clientdir.join("login").write(
-            json.dumps(dict(user=user, password=password)))
-        os.umask(oldumask)
+        try:
+            self.clientdir.join("login").write(
+                json.dumps(dict(user=user, password=password)))
+        finally:
+            os.umask(oldumask)
 
     def delete_auth(self):
         loginpath = self.clientdir.join("login")
@@ -166,7 +173,7 @@ class Hub:
         try:
             cmd = self.args.mainloc.split(":")[0]
             if cmd not in ("devpi.use", "devpi.server"):
-                ensure_autoserver(self, current)
+                handle_autoserver(self, current)
         except AttributeError:
             pass
         return current
@@ -218,12 +225,14 @@ class Hub:
         msg = " ".join(map(str, msgs))
         self._tw.line(msg, **kwargs)
 
+    # semantic logging
     def debug(self, *msg):
-        if self.args.debug:
+        if self.args.debug and not self.quiet:
             self.line("[debug]", *msg)
 
     def error(self, *msg):
-        self.line(*msg, red=True)
+        if not self.quiet:
+            self.line(*msg, red=True)
 
     def fatal(self, *msg):
         msg = " ".join(map(str, msg))
@@ -231,7 +240,8 @@ class Hub:
         raise SystemExit(1)
 
     def info(self, *msg):
-        self.line(*msg, bold=True)
+        if not self.quiet:
+            self.line(*msg, bold=True)
 
     def out_json(self, data):
         self._tw.line(json.dumps(data, sort_keys=True, indent=4))
@@ -292,10 +302,14 @@ def add_generic_options(parser):
 
 @subcommand("devpi.use")
 def use(parser):
-    """ configure remote index and target venv for install activities. """
+    """ show or configure remote index and target venv for install
+    activities. """
+
     parser.add_argument("--venv", action="store", default=None,
         help="set virtual environment to use for install activities. "
              "specify '-' to unset it.")
+    parser.add_argument("--no-auto", action="store_true", dest="noauto",
+        help="don't start automatic server")
     parser.add_argument("--urls", action="store_true",
         help="show remote endpoint urls")
     parser.add_argument("--delete", action="store_true",
@@ -374,12 +388,20 @@ def upload(parser):
     parser.add_argument("--formats", default="sdist.tgz", action="store",
         help="comma separated list of build formats (passed to setup.py). "
              "Examples sdist.zip,bdist_egg,bdist_dumb.")
-    parser.add_argument("--dryrun",
+    parser.add_argument("--from-dir", action="store", default=None,
+        dest="fromdir",
+        help="upload all archive files from the specified directory")
+    parser.add_argument("--only-latest", action="store_true",
+        help="upload only latest version if multiple archives for a "
+             "package are found (only effective with --from-dir)")
+    parser.add_argument("--dry-run", dest="dryrun",
         action="store_true", default=None,
         help="don't perform any server-modifying actions")
-    parser.add_argument("--withdocs", action="store_true", default=None,
+    parser.add_argument("--with-docs", action="store_true", default=None,
+        dest="withdocs",
         help="perform upload_docs in addition to uploading release files")
-    parser.add_argument("--onlydocs", action="store_true", default=None,
+    parser.add_argument("--only-docs", action="store_true", default=None,
+        dest="onlydocs",
         help="perform only upload_docs and no release files")
     #parser.add_argument("-y", dest="yes",
     #    action="store_true", default=None,
@@ -437,8 +459,8 @@ def server(parser):
     """ commands for controling the automatic server. """
     parser.add_argument("--stop", action="store_true",
         help="stop automatically started server if any")
-    parser.add_argument("--nolog", action="store_true",
-        help="don't show log file when showing status")
+    parser.add_argument("--log", action="store_true",
+        help="show last log lines of automatic server")
 
 
 if __name__ == "__main__":
