@@ -1,4 +1,5 @@
 import os, sys
+import json
 import py
 import pytest
 import types
@@ -26,7 +27,6 @@ def uploadhub(request, tmpdir):
     return hub
 
 
-
 class TestCheckout:
     @pytest.fixture(scope="class")
     def repo(self, request):
@@ -133,21 +133,61 @@ def test_setuppy_execution_namespace(monkeypatch, tmpdir):
     run_setuppy()
 
 class TestUploadFunctional:
-    def test_dryrun_all(self, initproj, devpi):
+    def test_all(self, initproj, devpi):
         initproj("hello-1.0", {"doc": {
             "conf.py": "",
             "index.html": "<html/>"}})
         assert py.path.local("setup.py").check()
-        devpi("upload", "--dryrun")
-        devpi("upload", "--dryrun", "--withdocs")
-        devpi("upload", "--dryrun", "--onlydocs")
+        devpi("upload", "--dry-run")
+        devpi("upload", "--dry-run", "--with-docs")
+        devpi("upload", "--dry-run", "--only-docs")
         devpi("upload", "--formats", "sdist.tbz")
         devpi("upload", "--formats", "sdist.tgz,bdist_dumb")
 
         # logoff then upload
         devpi("logoff")
-        devpi("upload", "--dryrun", code=404)
+        devpi("upload", "--dry-run", code=404)
 
         # go to other index
         devpi("use", "root/pypi")
-        devpi("upload", "--dryrun", code=404)
+        devpi("upload", "--dry-run", code=404)
+
+    def test_fromdir(self, initproj, devpi, out_devpi, runproc):
+        initproj("hello-1.1", {"doc": {
+            "conf.py": "",
+            "index.html": "<html/>"}})
+        tmpdir = py.path.local()
+        runproc(tmpdir, "python setup.py sdist".split())
+        dist = tmpdir.join("dist")
+        assert dist.check()
+        hub = devpi("upload", "--from-dir", dist)
+        out = out_devpi("getjson", hub.current.index + "hello/1.1/")
+        data = json.loads(out.stdout.str())
+        if sys.platform == "win32":
+            assert "hello-1.1.zip" in data["result"]["+files"]
+        else:
+            assert "hello-1.1.tar.gz" in data["result"]["+files"]
+
+def test_minimal_pkginfo():
+    from devpi.upload.upload import MinimalPkgInfo
+    minpack = MinimalPkgInfo('this/is/a/test-abc-0.1.2dev3.tar.gz')
+    assert minpack.version == '0.1.2dev3'
+    assert verlib.normversion(minpack.version) == (u'00000000', u'00000001',
+                                            u'00000002', u'*@', u'00000003', '*final')
+
+
+def test_filter_latest():
+    from devpi.upload.upload import MinimalPkgInfo, filter_latest
+    d = {}
+    for idx in [1, 9, 10]:
+        path = 'this/is/a/test-abc-0.%d' % (idx)
+        d[path] = MinimalPkgInfo(path)
+    assert len(d) == 3
+    d = filter_latest(d)
+    assert len(d) == 1
+    filtered = d[path]
+    assert filtered.name == 'test-abc'
+    assert filtered.version == u'0.10'
+
+
+
